' Declarations for MPEG header class
' A few layer III, MPEG-2 LSF, and seeking modifications made by Jeff Tsay.
' Last modified : 04/19/97
'
'  header.h 1.7, last edit: 6/15/94 16:55:33
'  Copyright (C) 1993, 1994 Tobias Bading (bading@cs.tu-berlin.de)
'  Berlin University of Technology

' Class for extracting information from a frame header
Public Class FrameHeader

#Region " Public Constants "
    Public Const STEREO As Integer = 0
    Public Const JOINT_STEREO As Integer = 1
    Public Const DUAL_CHANNEL As Integer = 2
    Public Const SINGLE_CHANNEL As Integer = 3
    Public Const FOURTYFOUR_POINT_ONE As Integer = 0
    Public Const FOURTYEIGHT As Integer = 1
    Public Const THIRTYTWO As Integer = 2
#End Region

#Region " Private Variables "
    Private h_protection_bit, h_bitrate_index, h_padding_bit, h_mode_extension As Integer 'h_Layer must be 3
    Private h_mode As Integer
    Private h_sample_frequency As Integer
    Private h_number_of_subbands, h_intensity_stereo_bound As Integer
    Private h_copyright, h_original As Boolean
    Private m_syncmode As Short = BitStream.INITIAL_SYNC

    Private m_framesize As Integer
    Private m_Slots As Integer

    Private m_bitrates() As Integer = {0, 32000, 40000, 48000, 56000, 64000, 80000, 96000, 112000, 128000, 160000, 192000, 224000, 256000, 320000, 0}
    Private m_frequencies() As Integer = {44100, 48000, 32000, 1}
#End Region

#Region " Functions "
    ' Read a 32-bit header from the BitStream
    Friend Sub ReadHeader(ByVal m_bstream As Bitstream)
        Dim m_headerstring As Integer
        Dim channel_bitrate As Integer

        Dim sync As Boolean = False

        Do
            m_headerstring = m_bstream.SyncHeader(m_syncmode)

            If m_syncmode = Bitstream.INITIAL_SYNC Then
                h_sample_frequency = Common.URShift(m_headerstring, 10) And 3
                If h_sample_frequency = 3 Then
                    Throw m_bstream.NewBitstreamException(BitstreamErrors.UNKNOWN_ERROR)
                End If
            End If

            h_protection_bit = Common.URShift(m_headerstring, 16) And 1
            h_bitrate_index = Common.URShift(m_headerstring, 12) And &HF
            h_padding_bit = Common.URShift(m_headerstring, 9) And 1
            h_mode = Common.URShift(m_headerstring, 6) And 3
            h_mode_extension = Common.URShift(m_headerstring, 4) And 3
            If h_mode = JOINT_STEREO Then
                h_intensity_stereo_bound = (h_mode_extension << 2) + 4
            Else
                h_intensity_stereo_bound = 0
            End If
            If (Common.URShift(m_headerstring, 3) And 1) = 1 Then
                h_copyright = True
            End If
            If (Common.URShift(m_headerstring, 2) And 1) = 1 Then
                h_original = True
            End If

            channel_bitrate = h_bitrate_index
            ' calculate bitrate per channel
            If h_mode <> SINGLE_CHANNEL Then
                If channel_bitrate = 4 Then
                    channel_bitrate = 1
                Else
                    channel_bitrate -= 4
                End If
            End If
            If channel_bitrate = 1 OrElse channel_bitrate = 2 Then
                If h_sample_frequency = THIRTYTWO Then
                    h_number_of_subbands = 12
                Else
                    h_number_of_subbands = 8
                End If
            ElseIf h_sample_frequency = FOURTYEIGHT OrElse (channel_bitrate >= 3 AndAlso channel_bitrate <= 5) Then
                h_number_of_subbands = 27
            Else
                h_number_of_subbands = 30
            End If

            If h_intensity_stereo_bound > h_number_of_subbands Then
                h_intensity_stereo_bound = h_number_of_subbands
            End If
            ' calculate framesize and nSlots
            CalculateFramesize()
            ' read framedata
            m_bstream.ReadFrameData(m_framesize)

            If m_bstream.IsSyncCurrentPosition(m_syncmode) Then
                If m_syncmode = Bitstream.INITIAL_SYNC Then
                    m_syncmode = Bitstream.STRICT_SYNC
                    m_bstream.SetSyncword(m_headerstring And &HFFF80CC0)
                End If
                sync = True
            Else
                m_bstream.UnreadFrame()
            End If
        Loop While Not sync

        m_bstream.ParseFrame()

        If h_protection_bit = 0 Then
            ' frame contains a crc checksum
            m_bstream.GetBits(16)
        End If
        If h_sample_frequency = FOURTYFOUR_POINT_ONE Then
        End If

    End Sub

#Region " Functions to query header contents "

    ' Returns Sample Frequency
    Public Function SampleFrequency() As Integer
        Return h_sample_frequency
    End Function
    ' Returns Frequency
    Public Function Frequency() As Integer
        Return m_frequencies(h_sample_frequency)
    End Function
    ' Returns Mode
    Public Function Mode() As Integer
        Return h_mode
    End Function
 
#End Region

#Region " Seeking and layer 3 stuff "

    ' Returns Slots
    Public Function Slots() As Integer
        Return m_Slots
    End Function

    ' Returns Mode Extension
    Public Function ModeExtension() As Integer
        Return h_mode_extension
    End Function

    ' Calculates framesize in bytes excluding header size
    Public Function CalculateFramesize() As Integer
      
        m_framesize = 144 * m_bitrates(h_bitrate_index) \ m_frequencies(h_sample_frequency)
        
        If h_padding_bit <> 0 Then
            m_framesize += 1
        End If
        ' Layer 3 slots
        Dim protection_int As Integer
        Dim mode_int As Integer
        If h_protection_bit <> 0 Then
            protection_int = 0
        Else
            protection_int = 2
        End If

        If h_mode = SINGLE_CHANNEL Then
            mode_int = 17
        Else
            mode_int = 32
        End If
       
        m_Slots = m_framesize - mode_int - protection_int - 4 ' header size
        
        m_framesize -= 4 ' subtract header size
        Return m_framesize

    End Function

#End Region

#End Region

End Class