' date          programmers         comment                               
'
' 16/11/99      mdm@techie.com      Renamed class, added javadoc, and changed table name from String to 3 chars. 
' 02/15/99      E.B (ebsp@iname.com)Java Conversion 
'               javalayer@javazoom.net
'                                   
' 04/19/97                          Adapted from the ISO MPEG Audio Subgroup Software Simulation  Group's public 
'                                   c source for for its MPEG audio decoder. Miscellaneous changes by Jeff Tsay 
'                                   (ctsay@pasteur.eecs.berkeley.edu).
'
' ----------------------------------------------------------------------------------------------------------
' Copyright (c) 1991 MPEG/audio software simulation group, All Rights Reserved
'   MPEG/audio coding/decoding software, work in progress             
'    NOT for public distribution until verified and approved by the MPEG/audio committee.
'    For further information, please contact Davis Pan, 508-493-2241, e-mail: pan@3d.enet.dec.com            
'                                                                    
'   VERSION 4.1                                                        
'    changes made since last update:                                  
' date          programmers         comment                               
' 02/27/92      F.O.Witte (ITT Intermetall)				                  
' 08/24/93      M. Iwadare          Changed for 1 pass decoding.          
' 07/14/94      J. Koller           useless 'typedef' before huffcodetab removed				      			                  *
' ----------------------------------------------------------------------------------------------------------



' Class Implementing Huffcode Table.
Public Class HuffcodeTable

    Private m_tablename0 As Char    ' containing table_description
    Private m_tablename1 As Char    ' containing table_description

    Private xlaenge As Integer      ' max. x-index + 1
    Private ylaenge As Integer      ' max. y-index + 1
    Private m_linbits As Integer    ' number of linbits
    Private m_val As Integer()()    ' decoder tree
    Private m_treelen As Integer    ' length of decoder tree

    ' Constructor
    Public Sub New(ByVal S As String, ByVal XLen As Integer, ByVal YLen As Integer, ByVal Linbits As Integer, ByVal Val()() As Integer, ByVal Treelen As Integer)
        m_tablename0 = S.Chars(0)
        m_tablename1 = S.Chars(1)
        xlaenge = XLen
        ylaenge = YLen
        m_linbits = Linbits
        m_val = Val
        m_treelen = Treelen
    End Sub

    ' Decoder Function
    ' Does the huffman-decoding.
    Public Function HuffmanDecode(ByRef m_x As Integer, ByRef m_y As Integer, ByRef m_v As Integer, ByRef m_w As Integer, ByVal br As BitReserve) As Integer

        Dim dmask As Integer = (1 << 31)
        Dim level As Integer
        Dim point As Integer
        Dim [error] As Integer = 1
        level = dmask

        If m_val Is Nothing Then
            Return 2
        End If

        ' table 0 needs no bits

        If m_treelen = 0 Then
            m_x = 0
            m_y = 0
            Return 0
        End If

        ' Lookup in Huffman table.

        Do
            If m_val(point)(0) = 0 Then
                ' end of tree
                m_x = Common.URShift(m_val(point)(1), 4)
                m_y = m_val(point)(1) And &HF
                [error] = 0
                Exit Do
            End If

            ' get1bit() is called thousands of times, and so needs to be ultra fast. 

            If br.Get1Bit() <> 0 Then
                While m_val(point)(1) >= 250
                    point += m_val(point)(1)
                End While
                point += m_val(point)(1)
            Else
                While m_val(point)(0) >= 250
                    point += m_val(point)(0)
                End While
                point += m_val(point)(0)
            End If
            level = Common.URShift(level, 1)
        Loop While level <> 0 OrElse point < 0

        ' put back any bits not consumed

        If m_tablename0 = "3"c AndAlso (m_tablename1 = "2"c OrElse m_tablename1 = "3"c) Then
            m_v = (m_y >> 3) And 1
            m_w = (m_y >> 2) And 1
            m_x = (m_y >> 1) And 1
            m_y = m_y And 1

            If m_v <> 0 Then
                If br.Get1Bit() <> 0 Then
                    m_v = -m_v
                End If
            End If
            If m_w <> 0 Then
                If br.Get1Bit() <> 0 Then
                    m_w = -m_w
                End If
            End If
            If m_x <> 0 Then
                If br.Get1Bit() <> 0 Then
                    m_x = -m_x
                End If
            End If
            If m_y <> 0 Then
                If br.Get1Bit() <> 0 Then
                    m_y = -m_y
                End If
            End If
        Else
            ' Process sign and escape encodings for dual tables.
            If m_linbits <> 0 Then
                If xlaenge - 1 = m_x Then
                    m_x += br.GetBits(m_linbits)
                End If
            End If
            If m_x <> 0 Then
                If br.Get1Bit() <> 0 Then
                    m_x = -m_x
                End If
            End If
            If m_linbits <> 0 Then
                If ylaenge - 1 = m_y Then
                    m_y += br.GetBits(m_linbits)
                End If
            End If
            If m_y <> 0 Then
                If br.Get1Bit() <> 0 Then
                    m_y = -m_y
                End If
            End If
        End If
        Return [error]

    End Function

End Class
