' A class for the synthesis filter bank
' This class does a fast downsampling from 32, 44.1 or 48 kHz to 8 kHz, if ULAW is defined
' Frequencies above 4 kHz are removed by ignoring higher subbands
Public Class SynthesisFilter

#Region " Private Variables "
    Private v1(511) As Single
    Private v2(511) As Single
    Private m_actual_v() As Single          ' v1 or v2
    Private m_actual_write_pos As Integer   ' 0-15
    Private m_samples(31) As Single         ' 32 new subband samples
    Private m_channel As Integer
    Private m_scalefactor As Single
    Private m_eq(31) As Single
    Private m_tmpOut(31) As Single
    Private d16 As Single()()
#End Region

#Region " Constructor "
    ' The scalefactor scales the calculated float pcm samples to short values
    ' (raw pcm samples are in [-1.0, 1.0], if no violations occur)
    Public Sub New(ByVal channelnumber As Integer, ByVal factor As Single)
        d16 = SplitArray(d_data, 16)

        m_channel = channelnumber
        m_scalefactor = factor

        For i As Integer = 0 To 31
            m_eq(i) = 1.0F
        Next i

        Reset()
    End Sub
#End Region

#Region " Functions "
    ' Reset the synthesis filter
    Public Sub Reset()

        For p As Integer = 0 To 511
            v1(p) = 0.0F
            v2(p) = 0.0F
        Next p

        For p As Integer = 0 To 31
            m_samples(p) = 0.0F
        Next p

        m_actual_v = v1
        m_actual_write_pos = 15
    End Sub

    Public Sub InputSamples(ByVal s() As Single)
        For i As Integer = 0 To 31
            m_samples(i) = s(i) * m_eq(i)
        Next i
    End Sub

    ' Compute new values via a fast cosine transform
    Private Sub compute_new_v()
        Dim new_v0, new_v1, new_v2, new_v3, new_v4, new_v5, new_v6, new_v7, new_v8, new_v9, _
         new_v10, new_v11, new_v12, new_v13, new_v14, new_v15, new_v16, new_v17, new_v18, new_v19, _
         new_v20, new_v21, new_v22, new_v23, new_v24, new_v25, new_v26, new_v27, new_v28, new_v29, _
         new_v30, new_v31 As Single

        new_v0 = 0.0F
        new_v1 = 0.0F
        new_v2 = 0.0F
        new_v3 = 0.0F
        new_v4 = 0.0F
        new_v5 = 0.0F
        new_v6 = 0.0F
        new_v7 = 0.0F
        new_v8 = 0.0F
        new_v9 = 0.0F
        new_v10 = 0.0F
        new_v11 = 0.0F
        new_v12 = 0.0F
        new_v13 = 0.0F
        new_v14 = 0.0F
        new_v15 = 0.0F
        new_v16 = 0.0F
        new_v17 = 0.0F
        new_v18 = 0.0F
        new_v19 = 0.0F
        new_v20 = 0.0F
        new_v21 = 0.0F
        new_v22 = 0.0F
        new_v23 = 0.0F
        new_v24 = 0.0F
        new_v25 = 0.0F
        new_v26 = 0.0F
        new_v27 = 0.0F
        new_v28 = 0.0F
        new_v29 = 0.0F
        new_v30 = 0.0F
        new_v31 = 0.0F

        Dim s As Single() = m_samples

        Dim s0 As Single = s(0)
        Dim s1 As Single = s(1)
        Dim s2 As Single = s(2)
        Dim s3 As Single = s(3)
        Dim s4 As Single = s(4)
        Dim s5 As Single = s(5)
        Dim s6 As Single = s(6)
        Dim s7 As Single = s(7)
        Dim s8 As Single = s(8)
        Dim s9 As Single = s(9)
        Dim s10 As Single = s(10)
        Dim s11 As Single = s(11)
        Dim s12 As Single = s(12)
        Dim s13 As Single = s(13)
        Dim s14 As Single = s(14)
        Dim s15 As Single = s(15)
        Dim s16 As Single = s(16)
        Dim s17 As Single = s(17)
        Dim s18 As Single = s(18)
        Dim s19 As Single = s(19)
        Dim s20 As Single = s(20)
        Dim s21 As Single = s(21)
        Dim s22 As Single = s(22)
        Dim s23 As Single = s(23)
        Dim s24 As Single = s(24)
        Dim s25 As Single = s(25)
        Dim s26 As Single = s(26)
        Dim s27 As Single = s(27)
        Dim s28 As Single = s(28)
        Dim s29 As Single = s(29)
        Dim s30 As Single = s(30)
        Dim s31 As Single = s(31)

        Dim p0 As Single = s0 + s31
        Dim p1 As Single = s1 + s30
        Dim p2 As Single = s2 + s29
        Dim p3 As Single = s3 + s28
        Dim p4 As Single = s4 + s27
        Dim p5 As Single = s5 + s26
        Dim p6 As Single = s6 + s25
        Dim p7 As Single = s7 + s24
        Dim p8 As Single = s8 + s23
        Dim p9 As Single = s9 + s22
        Dim p10 As Single = s10 + s21
        Dim p11 As Single = s11 + s20
        Dim p12 As Single = s12 + s19
        Dim p13 As Single = s13 + s18
        Dim p14 As Single = s14 + s17
        Dim p15 As Single = s15 + s16

        Dim pp0 As Single = p0 + p15
        Dim pp1 As Single = p1 + p14
        Dim pp2 As Single = p2 + p13
        Dim pp3 As Single = p3 + p12
        Dim pp4 As Single = p4 + p11
        Dim pp5 As Single = p5 + p10
        Dim pp6 As Single = p6 + p9
        Dim pp7 As Single = p7 + p8
        Dim pp8 As Single = (p0 - p15) * cos1_32
        Dim pp9 As Single = (p1 - p14) * cos3_32
        Dim pp10 As Single = (p2 - p13) * cos5_32
        Dim pp11 As Single = (p3 - p12) * cos7_32
        Dim pp12 As Single = (p4 - p11) * cos9_32
        Dim pp13 As Single = (p5 - p10) * cos11_32
        Dim pp14 As Single = (p6 - p9) * cos13_32
        Dim pp15 As Single = (p7 - p8) * cos15_32

        p0 = pp0 + pp7
        p1 = pp1 + pp6
        p2 = pp2 + pp5
        p3 = pp3 + pp4
        p4 = (pp0 - pp7) * cos1_16
        p5 = (pp1 - pp6) * cos3_16
        p6 = (pp2 - pp5) * cos5_16
        p7 = (pp3 - pp4) * cos7_16
        p8 = pp8 + pp15
        p9 = pp9 + pp14
        p10 = pp10 + pp13
        p11 = pp11 + pp12
        p12 = (pp8 - pp15) * cos1_16
        p13 = (pp9 - pp14) * cos3_16
        p14 = (pp10 - pp13) * cos5_16
        p15 = (pp11 - pp12) * cos7_16


        pp0 = p0 + p3
        pp1 = p1 + p2
        pp2 = (p0 - p3) * cos1_8
        pp3 = (p1 - p2) * cos3_8
        pp4 = p4 + p7
        pp5 = p5 + p6
        pp6 = (p4 - p7) * cos1_8
        pp7 = (p5 - p6) * cos3_8
        pp8 = p8 + p11
        pp9 = p9 + p10
        pp10 = (p8 - p11) * cos1_8
        pp11 = (p9 - p10) * cos3_8
        pp12 = p12 + p15
        pp13 = p13 + p14
        pp14 = (p12 - p15) * cos1_8
        pp15 = (p13 - p14) * cos3_8

        p0 = pp0 + pp1
        p1 = (pp0 - pp1) * cos1_4
        p2 = pp2 + pp3
        p3 = (pp2 - pp3) * cos1_4
        p4 = pp4 + pp5
        p5 = (pp4 - pp5) * cos1_4
        p6 = pp6 + pp7
        p7 = (pp6 - pp7) * cos1_4
        p8 = pp8 + pp9
        p9 = (pp8 - pp9) * cos1_4
        p10 = pp10 + pp11
        p11 = (pp10 - pp11) * cos1_4
        p12 = pp12 + pp13
        p13 = (pp12 - pp13) * cos1_4
        p14 = pp14 + pp15
        p15 = (pp14 - pp15) * cos1_4

        Dim tmp1 As Single
        new_v12 = p7
        new_v4 = new_v12 + p5
        new_v19 = -new_v4 - p6
        new_v27 = -p6 - p7 - p4

        new_v14 = p15
        new_v10 = new_v14 + p11
        new_v6 = new_v10 + p13

        new_v2 = p15 + p13 + p9
        new_v17 = -new_v2 - p14

        tmp1 = -p14 - p15 - p10 - p11
        new_v21 = tmp1 - p13
        new_v29 = -p14 - p15 - p12 - p8
        new_v25 = tmp1 - p12
        new_v31 = -p0
        new_v0 = p1

        new_v8 = p3
        new_v23 = -new_v8 - p2

        p0 = (s0 - s31) * cos1_64
        p1 = (s1 - s30) * cos3_64
        p2 = (s2 - s29) * cos5_64
        p3 = (s3 - s28) * cos7_64
        p4 = (s4 - s27) * cos9_64
        p5 = (s5 - s26) * cos11_64
        p6 = (s6 - s25) * cos13_64
        p7 = (s7 - s24) * cos15_64
        p8 = (s8 - s23) * cos17_64
        p9 = (s9 - s22) * cos19_64
        p10 = (s10 - s21) * cos21_64
        p11 = (s11 - s20) * cos23_64
        p12 = (s12 - s19) * cos25_64
        p13 = (s13 - s18) * cos27_64
        p14 = (s14 - s17) * cos29_64
        p15 = (s15 - s16) * cos31_64


        pp0 = p0 + p15
        pp1 = p1 + p14
        pp2 = p2 + p13
        pp3 = p3 + p12
        pp4 = p4 + p11
        pp5 = p5 + p10
        pp6 = p6 + p9
        pp7 = p7 + p8
        pp8 = (p0 - p15) * cos1_32
        pp9 = (p1 - p14) * cos3_32
        pp10 = (p2 - p13) * cos5_32
        pp11 = (p3 - p12) * cos7_32
        pp12 = (p4 - p11) * cos9_32
        pp13 = (p5 - p10) * cos11_32
        pp14 = (p6 - p9) * cos13_32
        pp15 = (p7 - p8) * cos15_32


        p0 = pp0 + pp7
        p1 = pp1 + pp6
        p2 = pp2 + pp5
        p3 = pp3 + pp4
        p4 = (pp0 - pp7) * cos1_16
        p5 = (pp1 - pp6) * cos3_16
        p6 = (pp2 - pp5) * cos5_16
        p7 = (pp3 - pp4) * cos7_16
        p8 = pp8 + pp15
        p9 = pp9 + pp14
        p10 = pp10 + pp13
        p11 = pp11 + pp12
        p12 = (pp8 - pp15) * cos1_16
        p13 = (pp9 - pp14) * cos3_16
        p14 = (pp10 - pp13) * cos5_16
        p15 = (pp11 - pp12) * cos7_16


        pp0 = p0 + p3
        pp1 = p1 + p2
        pp2 = (p0 - p3) * cos1_8
        pp3 = (p1 - p2) * cos3_8
        pp4 = p4 + p7
        pp5 = p5 + p6
        pp6 = (p4 - p7) * cos1_8
        pp7 = (p5 - p6) * cos3_8
        pp8 = p8 + p11
        pp9 = p9 + p10
        pp10 = (p8 - p11) * cos1_8
        pp11 = (p9 - p10) * cos3_8
        pp12 = p12 + p15
        pp13 = p13 + p14
        pp14 = (p12 - p15) * cos1_8
        pp15 = (p13 - p14) * cos3_8


        p0 = pp0 + pp1
        p1 = (pp0 - pp1) * cos1_4
        p2 = pp2 + pp3
        p3 = (pp2 - pp3) * cos1_4
        p4 = pp4 + pp5
        p5 = (pp4 - pp5) * cos1_4
        p6 = pp6 + pp7
        p7 = (pp6 - pp7) * cos1_4
        p8 = pp8 + pp9
        p9 = (pp8 - pp9) * cos1_4
        p10 = pp10 + pp11
        p11 = (pp10 - pp11) * cos1_4
        p12 = pp12 + pp13
        p13 = (pp12 - pp13) * cos1_4
        p14 = pp14 + pp15
        p15 = (pp14 - pp15) * cos1_4

        Dim tmp2 As Single

        new_v15 = p15
        new_v13 = new_v15 + p7
        new_v11 = new_v13 + p11
        new_v5 = new_v11 + p5 + p13

        new_v9 = p15 + p11 + p3
        new_v7 = new_v9 + p13

        tmp1 = p13 + p15 + p9
        new_v1 = tmp1 + p1
        new_v16 = -new_v1 - p14

        new_v3 = tmp1 + p5 + p7
        new_v18 = -new_v3 - p6 - p14

        tmp1 = -p10 - p11 - p14 - p15
        new_v22 = tmp1 - p13 - p2 - p3
        new_v20 = tmp1 - p13 - p5 - p6 - p7
        new_v24 = tmp1 - p12 - p2 - p3

        tmp2 = p4 + p6 + p7
        new_v26 = tmp1 - p12 - tmp2

        tmp1 = -p8 - p12 - p14 - p15
        new_v30 = tmp1 - p0
        new_v28 = tmp1 - tmp2

        Dim dest As Single() = m_actual_v

        Dim pos As Integer = m_actual_write_pos

        dest((0 + pos)) = new_v0
        dest((16 + pos)) = new_v1
        dest((32 + pos)) = new_v2
        dest((48 + pos)) = new_v3
        dest((64 + pos)) = new_v4
        dest((80 + pos)) = new_v5
        dest((96 + pos)) = new_v6
        dest((112 + pos)) = new_v7
        dest((128 + pos)) = new_v8
        dest((144 + pos)) = new_v9
        dest((160 + pos)) = new_v10
        dest((176 + pos)) = new_v11
        dest((192 + pos)) = new_v12
        dest((208 + pos)) = new_v13
        dest((224 + pos)) = new_v14
        dest((240 + pos)) = new_v15

        dest((256 + pos)) = 0.0F

        dest((272 + pos)) = -new_v15
        dest((288 + pos)) = -new_v14
        dest((304 + pos)) = -new_v13
        dest((320 + pos)) = -new_v12
        dest((336 + pos)) = -new_v11
        dest((352 + pos)) = -new_v10
        dest((368 + pos)) = -new_v9
        dest((384 + pos)) = -new_v8
        dest((400 + pos)) = -new_v7
        dest((416 + pos)) = -new_v6
        dest((432 + pos)) = -new_v5
        dest((448 + pos)) = -new_v4
        dest((464 + pos)) = -new_v3
        dest((480 + pos)) = -new_v2
        dest((496 + pos)) = -new_v1

        If m_actual_v Is v1 Then
            dest = v2
        Else
            dest = v1
        End If
        dest((0 + pos)) = -new_v0
        dest((16 + pos)) = new_v16
        dest((32 + pos)) = new_v17
        dest((48 + pos)) = new_v18
        dest((64 + pos)) = new_v19
        dest((80 + pos)) = new_v20
        dest((96 + pos)) = new_v21
        dest((112 + pos)) = new_v22
        dest((128 + pos)) = new_v23
        dest((144 + pos)) = new_v24
        dest((160 + pos)) = new_v25
        dest((176 + pos)) = new_v26
        dest((192 + pos)) = new_v27
        dest((208 + pos)) = new_v28
        dest((224 + pos)) = new_v29
        dest((240 + pos)) = new_v30
        dest((256 + pos)) = new_v31

        dest((272 + pos)) = new_v30
        dest((288 + pos)) = new_v29
        dest((304 + pos)) = new_v28
        dest((320 + pos)) = new_v27
        dest((336 + pos)) = new_v26
        dest((352 + pos)) = new_v25
        dest((368 + pos)) = new_v24
        dest((384 + pos)) = new_v23
        dest((400 + pos)) = new_v22
        dest((416 + pos)) = new_v21
        dest((432 + pos)) = new_v20
        dest((448 + pos)) = new_v19
        dest((464 + pos)) = new_v18
        dest((480 + pos)) = new_v17
        dest((496 + pos)) = new_v16
    End Sub

    Private Sub compute_pcm_samples0()
        Dim vp As Single() = m_actual_v
        Dim tmpOut As Single() = m_tmpOut
        Dim dvp As Integer

        Dim i As Integer
        For i = 0 To 31
            Dim pcm_sample As Single
            Dim dp As Single() = d16(i)
            pcm_sample = (vp((0 + dvp)) * dp(0) + vp((15 + dvp)) * dp(1) + vp((14 + dvp)) * dp(2) + vp((13 + dvp)) * dp(3) + vp((12 + dvp)) * dp(4) + vp((11 + dvp)) * dp(5) + vp((10 + dvp)) * dp(6) + vp((9 + dvp)) * dp(7) + vp((8 + dvp)) * dp(8) + vp((7 + dvp)) * dp(9) + vp((6 + dvp)) * dp(10) + vp((5 + dvp)) * dp(11) + vp((4 + dvp)) * dp(12) + vp((3 + dvp)) * dp(13) + vp((2 + dvp)) * dp(14) + vp((1 + dvp)) * dp(15)) * m_scalefactor

            tmpOut(i) = pcm_sample

            dvp += 16
        Next i
    End Sub

    Private Sub compute_pcm_samples1()
        Dim vp As Single() = m_actual_v
        Dim tmpOut As Single() = m_tmpOut
        Dim dvp As Integer

        Dim i As Integer
        For i = 0 To 31
            Dim dp As Single() = d16(i)
            Dim pcm_sample As Single

            pcm_sample = (vp((1 + dvp)) * dp(0) + vp((0 + dvp)) * dp(1) + vp((15 + dvp)) * dp(2) + vp((14 + dvp)) * dp(3) + vp((13 + dvp)) * dp(4) + vp((12 + dvp)) * dp(5) + vp((11 + dvp)) * dp(6) + vp((10 + dvp)) * dp(7) + vp((9 + dvp)) * dp(8) + vp((8 + dvp)) * dp(9) + vp((7 + dvp)) * dp(10) + vp((6 + dvp)) * dp(11) + vp((5 + dvp)) * dp(12) + vp((4 + dvp)) * dp(13) + vp((3 + dvp)) * dp(14) + vp((2 + dvp)) * dp(15)) * m_scalefactor

            tmpOut(i) = pcm_sample

            dvp += 16
        Next i
    End Sub

    Private Sub compute_pcm_samples2()
        Dim vp As Single() = m_actual_v

        Dim tmpOut As Single() = m_tmpOut
        Dim dvp As Integer

        Dim i As Integer
        For i = 0 To 31
            Dim dp As Single() = d16(i)
            Dim pcm_sample As Single

            pcm_sample = (vp((2 + dvp)) * dp(0) + vp((1 + dvp)) * dp(1) + vp((0 + dvp)) * dp(2) + vp((15 + dvp)) * dp(3) + vp((14 + dvp)) * dp(4) + vp((13 + dvp)) * dp(5) + vp((12 + dvp)) * dp(6) + vp((11 + dvp)) * dp(7) + vp((10 + dvp)) * dp(8) + vp((9 + dvp)) * dp(9) + vp((8 + dvp)) * dp(10) + vp((7 + dvp)) * dp(11) + vp((6 + dvp)) * dp(12) + vp((5 + dvp)) * dp(13) + vp((4 + dvp)) * dp(14) + vp((3 + dvp)) * dp(15)) * m_scalefactor

            tmpOut(i) = pcm_sample

            dvp += 16
        Next i

    End Sub

    Private Sub compute_pcm_samples3()
        Dim vp As Single() = m_actual_v

        Dim tmpOut As Single() = m_tmpOut
        Dim dvp As Integer

        Dim i As Integer
        For i = 0 To 31
            Dim dp As Single() = d16(i)
            Dim pcm_sample As Single

            pcm_sample = (vp((3 + dvp)) * dp(0) + vp((2 + dvp)) * dp(1) + vp((1 + dvp)) * dp(2) + vp((0 + dvp)) * dp(3) + vp((15 + dvp)) * dp(4) + vp((14 + dvp)) * dp(5) + vp((13 + dvp)) * dp(6) + vp((12 + dvp)) * dp(7) + vp((11 + dvp)) * dp(8) + vp((10 + dvp)) * dp(9) + vp((9 + dvp)) * dp(10) + vp((8 + dvp)) * dp(11) + vp((7 + dvp)) * dp(12) + vp((6 + dvp)) * dp(13) + vp((5 + dvp)) * dp(14) + vp((4 + dvp)) * dp(15)) * m_scalefactor

            tmpOut(i) = pcm_sample

            dvp += 16
        Next i

    End Sub

    Private Sub compute_pcm_samples4()
        Dim vp As Single() = m_actual_v
        Dim tmpOut As Single() = m_tmpOut
        Dim dvp As Integer
        Dim i As Integer
        For i = 0 To 31
            Dim dp As Single() = d16(i)
            Dim pcm_sample As Single

            pcm_sample = (vp((4 + dvp)) * dp(0) + vp((3 + dvp)) * dp(1) + vp((2 + dvp)) * dp(2) + vp((1 + dvp)) * dp(3) + vp((0 + dvp)) * dp(4) + vp((15 + dvp)) * dp(5) + vp((14 + dvp)) * dp(6) + vp((13 + dvp)) * dp(7) + vp((12 + dvp)) * dp(8) + vp((11 + dvp)) * dp(9) + vp((10 + dvp)) * dp(10) + vp((9 + dvp)) * dp(11) + vp((8 + dvp)) * dp(12) + vp((7 + dvp)) * dp(13) + vp((6 + dvp)) * dp(14) + vp((5 + dvp)) * dp(15)) * m_scalefactor

            tmpOut(i) = pcm_sample

            dvp += 16
        Next i

    End Sub

    Private Sub compute_pcm_samples5()
        Dim vp As Single() = m_actual_v
        Dim tmpOut As Single() = m_tmpOut
        Dim dvp As Integer
        Dim i As Integer
        For i = 0 To 31
            Dim dp As Single() = d16(i)
            Dim pcm_sample As Single

            pcm_sample = (vp((5 + dvp)) * dp(0) + vp((4 + dvp)) * dp(1) + vp((3 + dvp)) * dp(2) + vp((2 + dvp)) * dp(3) + vp((1 + dvp)) * dp(4) + vp((0 + dvp)) * dp(5) + vp((15 + dvp)) * dp(6) + vp((14 + dvp)) * dp(7) + vp((13 + dvp)) * dp(8) + vp((12 + dvp)) * dp(9) + vp((11 + dvp)) * dp(10) + vp((10 + dvp)) * dp(11) + vp((9 + dvp)) * dp(12) + vp((8 + dvp)) * dp(13) + vp((7 + dvp)) * dp(14) + vp((6 + dvp)) * dp(15)) * m_scalefactor

            tmpOut(i) = pcm_sample

            dvp += 16
        Next i

    End Sub

    Private Sub compute_pcm_samples6()
        Dim vp As Single() = m_actual_v
        Dim tmpOut As Single() = m_tmpOut
        Dim dvp As Integer
        Dim i As Integer
        For i = 0 To 31
            Dim dp As Single() = d16(i)
            Dim pcm_sample As Single

            pcm_sample = (vp((6 + dvp)) * dp(0) + vp((5 + dvp)) * dp(1) + vp((4 + dvp)) * dp(2) + vp((3 + dvp)) * dp(3) + vp((2 + dvp)) * dp(4) + vp((1 + dvp)) * dp(5) + vp((0 + dvp)) * dp(6) + vp((15 + dvp)) * dp(7) + vp((14 + dvp)) * dp(8) + vp((13 + dvp)) * dp(9) + vp((12 + dvp)) * dp(10) + vp((11 + dvp)) * dp(11) + vp((10 + dvp)) * dp(12) + vp((9 + dvp)) * dp(13) + vp((8 + dvp)) * dp(14) + vp((7 + dvp)) * dp(15)) * m_scalefactor

            tmpOut(i) = pcm_sample

            dvp += 16
        Next i

    End Sub

    Private Sub compute_pcm_samples7()
        Dim vp As Single() = m_actual_v
        Dim tmpOut As Single() = m_tmpOut
        Dim dvp As Integer
        Dim i As Integer
        For i = 0 To 31
            Dim dp As Single() = d16(i)
            Dim pcm_sample As Single

            pcm_sample = (vp((7 + dvp)) * dp(0) + vp((6 + dvp)) * dp(1) + vp((5 + dvp)) * dp(2) + vp((4 + dvp)) * dp(3) + vp((3 + dvp)) * dp(4) + vp((2 + dvp)) * dp(5) + vp((1 + dvp)) * dp(6) + vp((0 + dvp)) * dp(7) + vp((15 + dvp)) * dp(8) + vp((14 + dvp)) * dp(9) + vp((13 + dvp)) * dp(10) + vp((12 + dvp)) * dp(11) + vp((11 + dvp)) * dp(12) + vp((10 + dvp)) * dp(13) + vp((9 + dvp)) * dp(14) + vp((8 + dvp)) * dp(15)) * m_scalefactor

            tmpOut(i) = pcm_sample

            dvp += 16
        Next i

    End Sub

    Private Sub compute_pcm_samples8()
        Dim vp As Single() = m_actual_v

        Dim tmpOut As Single() = m_tmpOut
        Dim dvp As Integer

        Dim i As Integer
        For i = 0 To 31
            Dim dp As Single() = d16(i)
            Dim pcm_sample As Single

            pcm_sample = (vp((8 + dvp)) * dp(0) + vp((7 + dvp)) * dp(1) + vp((6 + dvp)) * dp(2) + vp((5 + dvp)) * dp(3) + vp((4 + dvp)) * dp(4) + vp((3 + dvp)) * dp(5) + vp((2 + dvp)) * dp(6) + vp((1 + dvp)) * dp(7) + vp((0 + dvp)) * dp(8) + vp((15 + dvp)) * dp(9) + vp((14 + dvp)) * dp(10) + vp((13 + dvp)) * dp(11) + vp((12 + dvp)) * dp(12) + vp((11 + dvp)) * dp(13) + vp((10 + dvp)) * dp(14) + vp((9 + dvp)) * dp(15)) * m_scalefactor

            tmpOut(i) = pcm_sample

            dvp += 16
        Next i

    End Sub

    Private Sub compute_pcm_samples9()
        Dim vp As Single() = m_actual_v

        Dim tmpOut As Single() = m_tmpOut
        Dim dvp As Integer

        Dim i As Integer
        For i = 0 To 31
            Dim dp As Single() = d16(i)
            Dim pcm_sample As Single

            pcm_sample = (vp((9 + dvp)) * dp(0) + vp((8 + dvp)) * dp(1) + vp((7 + dvp)) * dp(2) + vp((6 + dvp)) * dp(3) + vp((5 + dvp)) * dp(4) + vp((4 + dvp)) * dp(5) + vp((3 + dvp)) * dp(6) + vp((2 + dvp)) * dp(7) + vp((1 + dvp)) * dp(8) + vp((0 + dvp)) * dp(9) + vp((15 + dvp)) * dp(10) + vp((14 + dvp)) * dp(11) + vp((13 + dvp)) * dp(12) + vp((12 + dvp)) * dp(13) + vp((11 + dvp)) * dp(14) + vp((10 + dvp)) * dp(15)) * m_scalefactor

            tmpOut(i) = pcm_sample

            dvp += 16
        Next i

    End Sub

    Private Sub compute_pcm_samples10()
        Dim vp As Single() = m_actual_v
        Dim tmpOut As Single() = m_tmpOut
        Dim dvp As Integer

        Dim i As Integer
        For i = 0 To 31
            Dim dp As Single() = d16(i)
            Dim pcm_sample As Single

            pcm_sample = (vp((10 + dvp)) * dp(0) + vp((9 + dvp)) * dp(1) + vp((8 + dvp)) * dp(2) + vp((7 + dvp)) * dp(3) + vp((6 + dvp)) * dp(4) + vp((5 + dvp)) * dp(5) + vp((4 + dvp)) * dp(6) + vp((3 + dvp)) * dp(7) + vp((2 + dvp)) * dp(8) + vp((1 + dvp)) * dp(9) + vp((0 + dvp)) * dp(10) + vp((15 + dvp)) * dp(11) + vp((14 + dvp)) * dp(12) + vp((13 + dvp)) * dp(13) + vp((12 + dvp)) * dp(14) + vp((11 + dvp)) * dp(15)) * m_scalefactor

            tmpOut(i) = pcm_sample

            dvp += 16
        Next i

    End Sub

    Private Sub compute_pcm_samples11()
        Dim vp As Single() = m_actual_v

        Dim tmpOut As Single() = m_tmpOut
        Dim dvp As Integer

        Dim i As Integer
        For i = 0 To 31
            Dim dp As Single() = d16(i)
            Dim pcm_sample As Single

            pcm_sample = (vp((11 + dvp)) * dp(0) + vp((10 + dvp)) * dp(1) + vp((9 + dvp)) * dp(2) + vp((8 + dvp)) * dp(3) + vp((7 + dvp)) * dp(4) + vp((6 + dvp)) * dp(5) + vp((5 + dvp)) * dp(6) + vp((4 + dvp)) * dp(7) + vp((3 + dvp)) * dp(8) + vp((2 + dvp)) * dp(9) + vp((1 + dvp)) * dp(10) + vp((0 + dvp)) * dp(11) + vp((15 + dvp)) * dp(12) + vp((14 + dvp)) * dp(13) + vp((13 + dvp)) * dp(14) + vp((12 + dvp)) * dp(15)) * m_scalefactor

            tmpOut(i) = pcm_sample

            dvp += 16
        Next i

    End Sub

    Private Sub compute_pcm_samples12()
        Dim vp As Single() = m_actual_v
        Dim tmpOut As Single() = m_tmpOut
        Dim dvp As Integer

        Dim i As Integer
        For i = 0 To 31
            Dim dp As Single() = d16(i)
            Dim pcm_sample As Single

            pcm_sample = (vp((12 + dvp)) * dp(0) + vp((11 + dvp)) * dp(1) + vp((10 + dvp)) * dp(2) + vp((9 + dvp)) * dp(3) + vp((8 + dvp)) * dp(4) + vp((7 + dvp)) * dp(5) + vp((6 + dvp)) * dp(6) + vp((5 + dvp)) * dp(7) + vp((4 + dvp)) * dp(8) + vp((3 + dvp)) * dp(9) + vp((2 + dvp)) * dp(10) + vp((1 + dvp)) * dp(11) + vp((0 + dvp)) * dp(12) + vp((15 + dvp)) * dp(13) + vp((14 + dvp)) * dp(14) + vp((13 + dvp)) * dp(15)) * m_scalefactor

            tmpOut(i) = pcm_sample

            dvp += 16
        Next i

    End Sub

    Private Sub compute_pcm_samples13()
        Dim vp As Single() = m_actual_v

        Dim tmpOut As Single() = m_tmpOut
        Dim dvp As Integer

        Dim i As Integer
        For i = 0 To 31
            Dim dp As Single() = d16(i)
            Dim pcm_sample As Single

            pcm_sample = (vp((13 + dvp)) * dp(0) + vp((12 + dvp)) * dp(1) + vp((11 + dvp)) * dp(2) + vp((10 + dvp)) * dp(3) + vp((9 + dvp)) * dp(4) + vp((8 + dvp)) * dp(5) + vp((7 + dvp)) * dp(6) + vp((6 + dvp)) * dp(7) + vp((5 + dvp)) * dp(8) + vp((4 + dvp)) * dp(9) + vp((3 + dvp)) * dp(10) + vp((2 + dvp)) * dp(11) + vp((1 + dvp)) * dp(12) + vp((0 + dvp)) * dp(13) + vp((15 + dvp)) * dp(14) + vp((14 + dvp)) * dp(15)) * m_scalefactor

            tmpOut(i) = pcm_sample

            dvp += 16
        Next i

    End Sub

    Private Sub compute_pcm_samples14()
        Dim vp As Single() = m_actual_v
        Dim tmpOut As Single() = m_tmpOut
        Dim dvp As Integer
        Dim i As Integer
        For i = 0 To 31
            Dim dp As Single() = d16(i)
            Dim pcm_sample As Single

            pcm_sample = (vp((14 + dvp)) * dp(0) + vp((13 + dvp)) * dp(1) + vp((12 + dvp)) * dp(2) + vp((11 + dvp)) * dp(3) + vp((10 + dvp)) * dp(4) + vp((9 + dvp)) * dp(5) + vp((8 + dvp)) * dp(6) + vp((7 + dvp)) * dp(7) + vp((6 + dvp)) * dp(8) + vp((5 + dvp)) * dp(9) + vp((4 + dvp)) * dp(10) + vp((3 + dvp)) * dp(11) + vp((2 + dvp)) * dp(12) + vp((1 + dvp)) * dp(13) + vp((0 + dvp)) * dp(14) + vp((15 + dvp)) * dp(15)) * m_scalefactor

            tmpOut(i) = pcm_sample

            dvp += 16
        Next i

    End Sub

    Private Sub compute_pcm_samples15()
        Dim vp As Single() = m_actual_v

        Dim tmpOut As Single() = m_tmpOut
        Dim dvp As Integer

        Dim i As Integer
        For i = 0 To 31
            Dim pcm_sample As Single
            Dim dp As Single() = d16(i)
            pcm_sample = (vp((15 + dvp)) * dp(0) + vp((14 + dvp)) * dp(1) + vp((13 + dvp)) * dp(2) + vp((12 + dvp)) * dp(3) + vp((11 + dvp)) * dp(4) + vp((10 + dvp)) * dp(5) + vp((9 + dvp)) * dp(6) + vp((8 + dvp)) * dp(7) + vp((7 + dvp)) * dp(8) + vp((6 + dvp)) * dp(9) + vp((5 + dvp)) * dp(10) + vp((4 + dvp)) * dp(11) + vp((3 + dvp)) * dp(12) + vp((2 + dvp)) * dp(13) + vp((1 + dvp)) * dp(14) + vp((0 + dvp)) * dp(15)) * m_scalefactor

            tmpOut(i) = pcm_sample
            dvp += 16
        Next i

    End Sub

    Private Sub compute_pcm_samples(ByVal buffer As AudioOutputBuffer)
        Select Case m_actual_write_pos
            Case 0
                compute_pcm_samples0()

            Case 1
                compute_pcm_samples1()

            Case 2
                compute_pcm_samples2()

            Case 3
                compute_pcm_samples3()

            Case 4
                compute_pcm_samples4()

            Case 5
                compute_pcm_samples5()

            Case 6
                compute_pcm_samples6()

            Case 7
                compute_pcm_samples7()

            Case 8
                compute_pcm_samples8()

            Case 9
                compute_pcm_samples9()

            Case 10
                compute_pcm_samples10()

            Case 11
                compute_pcm_samples11()

            Case 12
                compute_pcm_samples12()

            Case 13
                compute_pcm_samples13()

            Case 14
                compute_pcm_samples14()

            Case 15
                compute_pcm_samples15()
        End Select

        If Not (buffer Is Nothing) Then
            buffer.AppendSamples(m_channel, m_tmpOut)
        End If

    End Sub

    Public Sub CalculatePcmSamples(ByVal buffer As AudioOutputBuffer)
        compute_new_v()
        compute_pcm_samples(buffer)

        m_actual_write_pos = m_actual_write_pos + 1 And &HF
        If m_actual_v Is v1 Then
            m_actual_v = v2
        Else
            m_actual_v = v1
        End If
        Dim p As Integer
        For p = 0 To 31
            m_samples(p) = 0.0F
        Next p

    End Sub
    ' Converts a 1D array into a number of smaller arrays. This is used
    ' to achieve offset + constant indexing into an array. Each sub-array
    ' represents a block of values of the original array. 
    ' array()   :   The array to split up into blocks.
    ' blockSize :	The size of the blocks to split the array into. This 
    '               must be an exact divisor of the length of the array, 
    '               or some data will be lost from the main array.
    ' returns   :	An array of arrays in which each element in the returned
    '               array will be of length blockSize
    Private Function SplitArray(ByVal array() As Single, ByVal blockSize As Integer) As Single()()
        Dim size As Integer = array.Length \ blockSize
        Dim split(size)() As Single
        Dim i As Integer

        While i < size
            split(i) = SubArray(array, i * blockSize, blockSize)
            i += 1
        End While
        Return split

    End Function

    ' Returns a subarray of an existing array.
    ' array()   :	The array to retrieve a subarea from.
    ' offs	    :   The offset in the array that corresponds to the first 
    '               index of the subarray.
    ' len       :	The number of indeces in the subarray.
    ' returns   :   The subarray, which may be of length 0.
    Private Function SubArray(ByVal array() As Single, ByVal offs As Integer, ByVal len As Integer) As Single()
        If offs + len > array.Length Then
            len = array.Length - offs
        End If

        If len < 0 Then
            len = 0
        End If
        Dim m_subarray(len) As Single
        Dim i As Integer

        While i < len
            m_subarray(i) = array((offs + i))
            i += 1
        End While

        Return m_subarray

    End Function

    Private Function d_data() As Single()
        Return New Single() { _
    0.0F, -0.000442505F, 0.003250122F, -0.007003784F, 0.0310821533F, -0.07862854F, 0.100311279F, -0.572036743F, _
1.144989F, 0.572036743F, 0.100311279F, 0.07862854F, 0.0310821533F, 0.007003784F, 0.003250122F, 0.000442505F, _
-0.000015259F, -0.000473022F, 0.003326416F, -0.007919312F, 0.0305175781F, -0.08418274F, 0.090927124F, -0.6002197F, _
1.14428711F, 0.543823242F, 0.1088562F, 0.07305908F, 0.03147888F, 0.006118774F, 0.003173828F, 0.000396729F, _
-0.000015259F, -0.000534058F, 0.003387451F, -0.008865356F, 0.0297851563F, -0.08970642F, 0.08068848F, -0.6282959F, _
1.14221191F, 0.515609741F, 0.116577148F, 0.06752014F, 0.03173828F, 0.0052948F, 0.003082275F, 0.000366211F, _
-0.000015259F, -0.000579834F, 0.003433228F, -0.009841919F, 0.0288848877F, -0.09516907F, 0.06959534F, -0.6562195F, _
1.13876343F, 0.487472534F, 0.123474121F, 0.06199646F, 0.0318450928F, 0.004486084F, 0.002990723F, 0.000320435F, _
-0.000015259F, -0.00062561F, 0.003463745F, -0.010848999F, 0.0278015137F, -0.100540161F, 0.0576171875F, -0.6839142F, _
1.13392639F, 0.459472656F, 0.129577637F, 0.0565338135F, 0.0318145752F, 0.003723145F, 0.00289917F, 0.000289917F, _
-0.000015259F, -0.000686646F, 0.003479004F, -0.0118865967F, 0.0265350342F, -0.1058197F, 0.0447845459F, -0.71131897F, _
1.12774658F, 0.431655884F, 0.1348877F, 0.0511322021F, 0.0316619873F, 0.003005981F, 0.002792358F, 0.000259399F, _
-0.000015259F, -0.000747681F, 0.003479004F, -0.0129394531F, 0.02508545F, -0.110946655F, 0.0310821533F, -0.7383728F, _
1.120224F, 0.404083252F, 0.139450073F, 0.0458374023F, 0.03138733F, 0.002334595F, 0.002685547F, 0.000244141F, _
-0.000030518F, -0.000808716F, 0.003463745F, -0.0140228271F, 0.0234222412F, -0.115921021F, 0.01651001F, -0.7650299F, _
1.1113739F, 0.376800537F, 0.143264771F, 0.0406341553F, 0.03100586F, 0.001693726F, 0.002578735F, 0.000213623F, _
-0.000030518F, -0.00088501F, 0.003417969F, -0.01512146F, 0.0215759277F, -0.120697021F, 0.001068115F, -0.791214F, _
1.10121155F, 0.349868774F, 0.1463623F, 0.03555298F, 0.0305328369F, 0.001098633F, 0.002456665F, 0.000198364F, _
-0.000030518F, -0.000961304F, 0.003372192F, -0.0162353516F, 0.01953125F, -0.1252594F, -0.0152282706F, -0.816864F, _
1.08978271F, 0.323318481F, 0.1487732F, 0.03060913F, 0.0299377441F, 0.000549316F, 0.002349854F, 0.000167847F, _
-0.000030518F, -0.001037598F, 0.00328064F, -0.0173492432F, 0.01725769F, -0.129562378F, -0.03237915F, -0.841949463F, _
1.07711792F, 0.2972107F, 0.150497437F, 0.0258178711F, 0.0292816162F, 0.000030518F, 0.002243042F, 0.000152588F, _
-0.000045776F, -0.001113892F, 0.003173828F, -0.0184631348F, 0.0148010254F, -0.1335907F, -0.0503540039F, -0.8663635F, _
1.06321716F, 0.2715912F, 0.151596069F, 0.0211792F, 0.0285339355F, -0.000442505F, 0.002120972F, 0.000137329F, _
-0.000045776F, -0.001205444F, 0.003051758F, -0.0195770264F, 0.0121154794F, -0.137298584F, -0.06916809F, -0.890090942F, _
1.04815674F, 0.246505737F, 0.152069092F, 0.016708374F, 0.02772522F, -0.000869751F, 0.00201416F, 0.00012207F, _
-0.000061035F, -0.001296997F, 0.002883911F, -0.020690918F, 0.009231567F, -0.140670776F, -0.0887756348F, -0.9130554F, _
1.03193665F, 0.221984863F, 0.15196228F, 0.0124206543F, 0.02684021F, -0.001266479F, 0.001907349F, 0.000106812F, _
-0.000061035F, -0.00138855F, 0.002700806F, -0.02178955F, 0.006134033F, -0.143676758F, -0.109161377F, -0.9351959F, _
1.01461792F, 0.198059082F, 0.151306152F, 0.00831604F, 0.0259094238F, -0.001617432F, 0.001785278F, 0.000106812F, _
-0.000076294F, -0.001480103F, 0.002487183F, -0.022857666F, 0.002822876F, -0.1462555F, -0.130310059F, -0.956481934F, _
0.996246338F, 0.174789429F, 0.150115967F, 0.004394531F, 0.0249328613F, -0.001937866F, 0.001693726F, 0.000091553F, _
-0.000076294F, -0.001586914F, 0.002227783F, -0.0239105225F, -0.000686646F, -0.148422241F, -0.152206421F, -0.9768524F, _
0.9768524F, 0.152206421F, 0.148422241F, 0.000686646F, 0.0239105225F, -0.002227783F, 0.001586914F, 0.000076294F, _
-0.000091553F, -0.001693726F, 0.001937866F, -0.0249328613F, -0.004394531F, -0.150115967F, -0.174789429F, -0.996246338F, _
0.956481934F, 0.130310059F, 0.1462555F, -0.002822876F, 0.022857666F, -0.002487183F, 0.001480103F, 0.000076294F, _
-0.000106812F, -0.001785278F, 0.001617432F, -0.0259094238F, -0.00831604F, -0.151306152F, -0.198059082F, -1.01461792F, _
0.9351959F, 0.109161377F, 0.143676758F, -0.006134033F, 0.02178955F, -0.002700806F, 0.00138855F, 0.000061035F, _
-0.000106812F, -0.001907349F, 0.001266479F, -0.02684021F, -0.0124206543F, -0.15196228F, -0.221984863F, -1.03193665F, _
0.9130554F, 0.0887756348F, 0.140670776F, -0.009231567F, 0.020690918F, -0.002883911F, 0.001296997F, 0.000061035F, _
-0.00012207F, -0.00201416F, 0.000869751F, -0.02772522F, -0.016708374F, -0.152069092F, -0.246505737F, -1.04815674F, _
0.890090942F, 0.06916809F, 0.137298584F, -0.0121154794F, 0.0195770264F, -0.003051758F, 0.001205444F, 0.000045776F, _
-0.000137329F, -0.002120972F, 0.000442505F, -0.0285339355F, -0.0211792F, -0.151596069F, -0.2715912F, -1.06321716F, _
0.8663635F, 0.0503540039F, 0.1335907F, -0.0148010254F, 0.0184631348F, -0.003173828F, 0.001113892F, 0.000045776F, _
-0.000152588F, -0.002243042F, -0.000030518F, -0.0292816162F, -0.0258178711F, -0.150497437F, -0.2972107F, -1.07711792F, _
0.841949463F, 0.03237915F, 0.129562378F, -0.01725769F, 0.0173492432F, -0.00328064F, 0.001037598F, 0.000030518F, _
-0.000167847F, -0.002349854F, -0.000549316F, -0.0299377441F, -0.03060913F, -0.1487732F, -0.323318481F, -1.08978271F, _
0.816864F, 0.0152282706F, 0.1252594F, -0.01953125F, 0.0162353516F, -0.003372192F, 0.000961304F, 0.000030518F, _
-0.000198364F, -0.002456665F, -0.001098633F, -0.0305328369F, -0.03555298F, -0.1463623F, -0.349868774F, -1.10121155F, _
0.791214F, -0.001068115F, 0.120697021F, -0.0215759277F, 0.01512146F, -0.003417969F, 0.00088501F, 0.000030518F, _
-0.000213623F, -0.002578735F, -0.001693726F, -0.03100586F, -0.0406341553F, -0.143264771F, -0.376800537F, -1.1113739F, _
0.7650299F, -0.01651001F, 0.115921021F, -0.0234222412F, 0.0140228271F, -0.003463745F, 0.000808716F, 0.000030518F, _
-0.000244141F, -0.002685547F, -0.002334595F, -0.03138733F, -0.0458374023F, -0.139450073F, -0.404083252F, -1.120224F, _
0.7383728F, -0.0310821533F, 0.110946655F, -0.02508545F, 0.0129394531F, -0.003479004F, 0.000747681F, 0.000015259F, _
-0.000259399F, -0.002792358F, -0.003005981F, -0.0316619873F, -0.0511322021F, -0.1348877F, -0.431655884F, -1.12774658F, _
0.71131897F, -0.0447845459F, 0.1058197F, -0.0265350342F, 0.0118865967F, -0.003479004F, 0.000686646F, 0.000015259F, _
-0.000289917F, -0.00289917F, -0.003723145F, -0.0318145752F, -0.0565338135F, -0.129577637F, -0.459472656F, -1.13392639F, _
0.6839142F, -0.0576171875F, 0.100540161F, -0.0278015137F, 0.010848999F, -0.003463745F, 0.00062561F, 0.000015259F, _
-0.000320435F, -0.002990723F, -0.004486084F, -0.0318450928F, -0.06199646F, -0.123474121F, -0.487472534F, -1.13876343F, _
0.6562195F, -0.06959534F, 0.09516907F, -0.0288848877F, 0.009841919F, -0.003433228F, 0.000579834F, 0.000015259F, _
-0.000366211F, -0.003082275F, -0.0052948F, -0.03173828F, -0.06752014F, -0.116577148F, -0.515609741F, -1.14221191F, _
0.6282959F, -0.08068848F, 0.08970642F, -0.0297851563F, 0.008865356F, -0.003387451F, 0.000534058F, 0.000015259F, _
-0.000396729F, -0.003173828F, -0.006118774F, -0.03147888F, -0.07305908F, -0.1088562F, -0.543823242F, -1.14428711F, _
0.6002197F, -0.090927124F, 0.08418274F, -0.0305175781F, 0.007919312F, -0.003326416F, 0.000473022F, 0.000015259F}
    End Function

#End Region

#Region " More Private Variables "
    Private cos1_64 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI / 64.0)))
    Private cos3_64 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 3.0 / 64.0)))
    Private cos5_64 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 5.0 / 64.0)))
    Private cos7_64 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 7.0 / 64.0)))
    Private cos9_64 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 9.0 / 64.0)))
    Private cos11_64 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 11.0 / 64.0)))
    Private cos13_64 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 13.0 / 64.0)))
    Private cos15_64 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 15.0 / 64.0)))
    Private cos17_64 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 17.0 / 64.0)))
    Private cos19_64 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 19.0 / 64.0)))
    Private cos21_64 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 21.0 / 64.0)))
    Private cos23_64 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 23.0 / 64.0)))
    Private cos25_64 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 25.0 / 64.0)))
    Private cos27_64 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 27.0 / 64.0)))
    Private cos29_64 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 29.0 / 64.0)))
    Private cos31_64 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 31.0 / 64.0)))
    Private cos1_32 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI / 32.0)))
    Private cos3_32 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 3.0 / 32.0)))
    Private cos5_32 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 5.0 / 32.0)))
    Private cos7_32 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 7.0 / 32.0)))
    Private cos9_32 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 9.0 / 32.0)))
    Private cos11_32 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 11.0 / 32.0)))
    Private cos13_32 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 13.0 / 32.0)))
    Private cos15_32 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 15.0 / 32.0)))
    Private cos1_16 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI / 16.0)))
    Private cos3_16 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 3.0 / 16.0)))
    Private cos5_16 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 5.0 / 16.0)))
    Private cos7_16 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 7.0 / 16.0)))
    Private cos1_8 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI / 8.0)))
    Private cos3_8 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI * 3.0 / 8.0)))
    Private cos1_4 As Single = System.Convert.ToSingle(1.0 / (2.0 * Math.Cos(Math.PI / 4.0)))
#End Region

End Class
