Imports System.IO
' Sample showing how to read through an MP3 file and obtain its contents as a PCM byte stream.
Public Class Example

#Region " Private Variables / Events "

    Public Event Ready()

    Private m_MP3Path As String
    Private m_WavePath As String
    Private m_Abort As Boolean
    Private mp_stream As Mp3Stream
    Private m_TS As TimeSpan
    Private m_TotalBytes As Integer
#End Region

#Region " Constructor "
    Public Sub New(ByVal MP3Path As String, ByVal WavePath As String)
        m_MP3Path = MP3Path
        m_WavePath = WavePath
    End Sub
#End Region

#Region " Properties "
    ReadOnly Property PercentDone() As Integer
        Get
            If Not IsNothing(mp_stream) Then
                Return (mp_stream.Position * 100) \ mp_stream.Length
            Else
                Return 0
            End If
        End Get
    End Property
    WriteOnly Property Abort() As Boolean
        Set(ByVal Value As Boolean)
            m_Abort = Value
        End Set
    End Property
    ReadOnly Property TotalBytes() As Integer
        Get
            Return m_TotalBytes
        End Get
    End Property
    ReadOnly Property TS() As TimeSpan
        Get
            Return m_TS
        End Get
    End Property
#End Region

#Region " Main function "
    'converts mp to wave in same Directory
    Public Sub Convert()
        Dim StartTime As DateTime = DateTime.Now

        ' delete current wave file
        If File.Exists(m_WavePath) Then
            File.Delete(m_WavePath)
        End If

        'Streams for reading MP3 and writing Wave File
        mp_stream = New Mp3Stream(m_MP3Path)                            'MP3-Stream
        Dim w_stream As New FileStream(m_WavePath, FileMode.CreateNew)  'Wave-Stream
        Dim w_writer As New BinaryWriter(w_stream)                      'Wave-Writer

        Dim numberOfPcmBytesToReadPerChunk As Integer = 512
        ' Create the buffer
        Dim buffer(numberOfPcmBytesToReadPerChunk - 1) As Byte

        Dim bytesReturned As Integer = -1

        ' read MP3 Stream and write to wave file
        While bytesReturned <> 0
            bytesReturned = mp_stream.Read(buffer, 0, buffer.Length)
            w_writer.Write(buffer)
            m_TotalBytes += bytesReturned
            ' if converting abort
            If m_Abort Then
                w_writer.Close()
                w_stream.Close()
                mp_stream.Close()

                If File.Exists(m_WavePath) Then
                    File.Delete(m_WavePath)
                End If

                Exit Sub
            End If
        End While

        ' move Position in wave stream to zero because  
        ' the RIFF header is at the begin of the file
        w_stream.Position = 0

        'Dim FileLength As Integer = totalBytes + 44 - 8
        'Dim FormatLength As Integer = 16
        'Dim FormatTag As Short = 1
        'Dim Channels As Short = 2
        'Dim AbtastRateProSekunde As Integer = mp_stream.Frequency
        'Dim BlockAlign As Short = 4
        'Dim BytesProSekunde As Integer = mp_stream.Frequency * 4
        'Dim BitsPerSample As Short = 16


        'write RIFF-Header at start of the wave stream
        Dim m_Encoder As New System.Text.ASCIIEncoding

        w_writer.Write(m_Encoder.GetBytes("RIFF"))
        w_writer.Write(m_TotalBytes + 36)                'FileLength
        w_writer.Write(m_Encoder.GetBytes("WAVE"))
        w_writer.Write(m_Encoder.GetBytes("fmt "))
        w_writer.Write(16)                              'FormatLength
        w_writer.Write(CShort(1))                       'FormatTag
        w_writer.Write(CShort(2))                       'Channels
        w_writer.Write(mp_stream.Frequency)             'AbtastRateProSekunde
        w_writer.Write(CInt(mp_stream.Frequency * 4))   'BytesProSekunde
        w_writer.Write(CShort(4))                       'BlockAlign
        w_writer.Write(CShort(16))                      'BitsPerSample
        w_writer.Write(m_Encoder.GetBytes("data"))
        w_writer.Write(m_TotalBytes)

        w_writer.Close()
        w_stream.Close()


        m_TS = (DateTime.Now.Subtract(StartTime))

        RaiseEvent Ready()
    End Sub
#End Region

End Class
