Imports System.IO
' A BackStream (such a beast doesn't exist in .NET's libraries to my knowledge)
Public Class BackStream

#Region " Private Variables "

    Private m_stream As Stream
    Private m_BackBufferSize As Integer
    Private m_NumForwardBytesInBuffer As Integer
    Private m_TempBytes() As Byte
    Private m_CircularBuffer As CircularByteBuffer

#End Region

#Region " Constructor "

    Public Sub New(ByVal s As Stream, ByVal backBufferSize As Integer)
        m_stream = s
        m_BackBufferSize = backBufferSize
        ReDim m_TempBytes(m_BackBufferSize)
        m_CircularBuffer = New CircularByteBuffer(m_BackBufferSize)
    End Sub

#End Region

#Region " Functions "

    Public Function Read(ByVal toRead() As Byte, ByVal offset As Integer, ByVal length As Integer) As Integer
        ' Read 
        Dim currentByte As Integer
        Dim canReadStream As Boolean = True
        While currentByte < length AndAlso canReadStream
            If m_NumForwardBytesInBuffer > 0 Then
                ' from mem
                m_NumForwardBytesInBuffer -= 1
                toRead((offset + currentByte)) = m_CircularBuffer.GetByte(m_NumForwardBytesInBuffer)
                currentByte += 1
            Else
                ' from stream
                Dim newBytes As Integer = length - currentByte
                Dim numRead As Integer = m_stream.Read(m_TempBytes, 0, newBytes)
                canReadStream = numRead >= newBytes

                For i As Integer = 0 To numRead - 1
                    m_CircularBuffer.Push(m_TempBytes(i))
                    toRead((offset + currentByte + i)) = m_TempBytes(i)
                Next
                currentByte += numRead
            End If
        End While
        Return currentByte

    End Function

    Public Sub UnRead(ByVal length As Integer)
        m_NumForwardBytesInBuffer += length
        If m_NumForwardBytesInBuffer > m_BackBufferSize Then
            Console.WriteLine("YOUR BACKSTREAM IS FISTED!")
        End If
    End Sub

    Public Sub Close()
        m_stream.Close()
    End Sub

#End Region

End Class