' Implementation of Bit Reservoir for Layer 3.
' The implementation stores single bits as a word in the buffer. If
' a bit is set, the corresponding word in the buffer will be non-zero.
' If a bit is clear, the corresponding word is zero. Although this
' may seem waseful, this can be a factor of two quicker than 
' packing 8 bits to a byte and extracting. 

Public Class BitReserve

#Region " Private Variables "
    Private m_offset, m_totbit, m_buf_byte_idx As Integer
    Private m_buf(32767) As Integer
#End Region

#Region " Functions "

    ' Return totbit Field.
    Public Function SSTell() As Integer
        Return m_totbit
    End Function

    ' Read a number bits from the bit stream.
    Public Function GetBits(ByVal N As Integer) As Integer
        m_totbit += N

        Dim val As Integer

        Dim pos As Integer = m_buf_byte_idx
        If pos + N < 32768 Then
            While N > 0
                val = (val << 1)
                If m_buf(pos) <> 0 Then
                    val = val Or 1
                Else
                    val = val Or 0
                End If
                pos += 1
                N -= 1
            End While
        Else
            While N > 0
                val = (val << 1)
                If m_buf(pos) <> 0 Then
                    val = val Or 1
                Else
                    val = val Or 0
                End If
                pos = pos + 1 And 32767
                N -= 1
            End While
        End If
        m_buf_byte_idx = pos
        Return val

    End Function

    ' Read 1 bit from the bit stream.
    Public Function Get1Bit() As Integer
        m_totbit += 1
        Dim val As Integer = m_buf(m_buf_byte_idx)
        m_buf_byte_idx = m_buf_byte_idx + 1 And 32767
        Return val

    End Function

    ' Write 8 bits into the bit stream.
    Public Sub PutBuf(ByVal val As Integer)
        Dim ofs As Integer = m_offset
        m_buf(ofs) = val And &H80
        ofs += 1
        m_buf(ofs) = val And &H40
        ofs += 1
        m_buf(ofs) = val And &H20
        ofs += 1
        m_buf(ofs) = val And &H10
        ofs += 1
        m_buf(ofs) = val And &H8
        ofs += 1
        m_buf(ofs) = val And &H4
        ofs += 1
        m_buf(ofs) = val And &H2
        ofs += 1
        m_buf(ofs) = val And &H1
        ofs += 1

        If ofs = 32768 Then
            m_offset = 0
        Else
            m_offset = ofs
        End If

    End Sub

    ' Rewind N bits in Stream.
    Public Sub RewindNBits(ByVal N As Integer)
        m_totbit -= N
        m_buf_byte_idx -= N
        If m_buf_byte_idx < 0 Then
            m_buf_byte_idx += 32768
        End If

    End Sub

    ' Rewind N bytes in Stream.
    Public Sub RewindNBytes(ByVal N As Integer)
        Dim bits As Integer = (N << 3)
        m_totbit -= bits
        m_buf_byte_idx -= bits
        If m_buf_byte_idx < 0 Then
            m_buf_byte_idx += 32768
        End If

    End Sub

#End Region

End Class