' Internal class used to queue samples that are being obtained from an Mp3 stream. 
' This class handles stereo 16-bit data! Switch it out if you want mono or something.
Public Class OBuffer16BitStereo
    Inherits AudioOutputBuffer

#Region " Private Variables "
    ' Private Shared m_Channels As Integer = 2
    ' Write offset used in append_bytes
    Private m_buffer(4607) As Byte
    Private m_bufferp(1) As Integer
    ' Read offset used to read from the stream, in bytes.
    Private m_offset As Integer
    ' end marker, one past end of array. Same as bufferp[0], but without the array bounds check.
    Private m_end As Integer
#End Region

#Region " Properties "
    Public ReadOnly Property bytesLeft() As Integer
        Get
            Return m_end - m_offset
        End Get
    End Property
#End Region

#Region " Constructor "
    Public Sub New()
        ' Initialize the buffer pointers
        ClearBuffer()
    End Sub
#End Region

#Region " Public Functions "

    ' Copies as much of this buffer as will fit into hte output buffer.
    ' returns the amount of bytes copied.
    Public Function Read(ByVal buffer_out() As Byte, ByVal offset As Integer, ByVal count As Integer) As Integer
        Dim remaining As Integer = bytesLeft
        Dim copySize As Integer
        If count > remaining Then
            copySize = remaining
        Else
            ' Copy an even number of sample frames
            ' count Mod 2 * Channels
            Dim remainder As Integer = count Mod 4
            copySize = count - remainder
        End If

        Array.Copy(m_buffer, m_offset, buffer_out, offset, copySize)

        m_offset += copySize
        Return copySize

    End Function

    ' Inefficiently write one sample value
    Public Overrides Sub Append(ByVal channel As Integer, ByVal value As Short)
        m_buffer(m_bufferp(channel)) = System.Convert.ToByte(value And &HFF)
        m_buffer((m_bufferp(channel) + 1)) = System.Convert.ToByte((value >> 8))
        ' += Channels * 2
        m_bufferp(channel) += 4

    End Sub

    ' Efficiently write 32 samples
    Public Overrides Sub AppendSamples(ByVal channel As Integer, ByVal f() As Single)
        ' Always, 32 samples are appended
        Dim pos As Integer = m_bufferp(channel)

        Dim fs As Single
        For i As Integer = 0 To 31

            fs = f(i)
            If fs > 32767.0F Then
                fs = 32767.0F
            ElseIf fs < -32767.0F Then
                fs = -32767.0F
            End If
            Dim sample As Integer = Fix(fs)

            m_buffer(pos) = CByte(sample And &HFF)
            m_buffer((pos + 1)) = CByte(sample >> 8)
            ' += Channels * 2
            pos += 4
        Next

        m_bufferp(channel) = pos

    End Sub

    ' This implementation does not clear the buffer. 
    Public Overrides Sub ClearBuffer()
        m_offset = 0
        m_end = 0
        Dim i As Integer
        ' i < Channels
        While i < 2
            ' two bytes per channel
            m_bufferp(i) = i * 2
            i += 1
        End While
    End Sub

    Public Overrides Sub WriteBuffer(ByVal val As Integer)
        m_offset = 0
        m_end = m_bufferp(0)
    End Sub

#End Region

End Class